@@ -77,9 +77,12 @@ class DensityMatrixSimulator(BaseLocalSimulator):
                                 "cy",
                                 "cz",
                                 "ecr",
+                                "gpi",
+                                "gpi2",
                                 "h",
                                 "i",
                                 "iswap",
+                                "ms",
                                 "pswap",
                                 "phaseshift",
                                 "rx",
