@@ -147,9 +147,12 @@ class StateVectorSimulator(BaseLocalSimulator):
                             "cy",
                             "cz",
                             "ecr",
+                            "gpi",
+                            "gpi2",
                             "h",
                             "i",
                             "iswap",
+                            "ms",
                             "pswap",
                             "phaseshift",
                             "rx",
